'use strict';

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _optsDefaults = require('../optsDefaults');

var _optsDefaults2 = _interopRequireDefault(_optsDefaults);

var _validateOpts = require('./validateOpts');

var _makeRoot = require('./makeRoot');

var _makeDefaultStyles = require('./makeDefaultStyles');

var _makeThemeBaseStyles = require('./makeThemeBaseStyles');

var _getDefaultTheme = require('./utils/getDefaultTheme');

var _addMissingProps = require('./utils/addMissingProps');

var _makeThemes = require('./makeThemes');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const buildCSS = pluginOpts => new Promise(async (resolve, reject) => {
    try {
        const opts = await (0, _validateOpts.validateOpts)(pluginOpts);
        const defaultTheme = (0, _addMissingProps.addMissingProps)({
            theme: (0, _getDefaultTheme.getDefaultTheme)(opts),
            opts
        });

        resolve(`
${(0, _makeRoot.makeRoot)(defaultTheme)}
${(0, _makeDefaultStyles.makeDefaultStyles)(defaultTheme)}
${(0, _makeThemeBaseStyles.makeThemeBaseStyles)(opts.themes)}
${(0, _makeThemes.makeThemes)({ opts, themes: opts.themes })}
`);
    } catch (err) {
        reject(err);
    }
});

const customPropsThemes = (opts = {}) => {
    return (root, result) => new Promise((resolve, reject) => {
        if (root.source.input.css.indexOf('@custom-props-themes') === -1) {
            result.warn('@custom-props-themes at rule not found');
            resolve();
            return;
        }

        root.walkAtRules('custom-props-themes', async rule => {
            try {
                const css = await buildCSS(opts.length ? opts : _optsDefaults2.default);
                rule.before(css);
                rule.remove();
                resolve();
            } catch (err) {
                reject(err);
            }
        });
    });
};

module.exports = _postcss2.default.plugin('custom-props-themes', customPropsThemes);