'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.validateOpts = undefined;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ajv = require('ajv');

var _ajv2 = _interopRequireDefault(_ajv);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Validate options passed to the plugin.
 * @param {Object} pluginOpts The passed-in options.
 * @param {string} schemaFile A path to the file containing the schema.
 * @return {Promise} A promise resolving with validated options or an error on rejection.
 */
const validateOpts = exports.validateOpts = (opts, schemaFile = _path2.default.resolve(__dirname, '../optsSchema.json')) => new Promise((resolve, reject) => {
    _fs2.default.readFile(schemaFile, 'utf-8', (err, schemaBuffer) => {
        if (err) {
            reject(err);
            return;
        }

        const ajv = new _ajv2.default();
        const valid = ajv.validate(JSON.parse(schemaBuffer), opts);

        if (!valid) {
            reject(ajv.errorsText());
            return;
        }

        resolve(opts);
    });
});